<object width="{{w}}" height="{{h}}">
<param name="movie" value="{{video_url}}"></param>
<param name="allowFullScreen" value="true"></param>
<param name="allowscriptaccess" value="always"></param>
<embed src="{{video_url}}"
type="application/x-shockwave-flash" allowscriptaccess="always"
allowfullscreen="true" width="{{w}}" height="{{h}}"></embed>
</object>
